/***************************************************************************/
/*                                                                         */
/*  t1types.h                                                              */
/*                                                                         */
/*    Basic Type1/Type2 type definitions and interface (specification      */
/*    only).                                                               */
/*                                                                         */
/*  Copyright 1996-2001, 2002 by                                           */
/*  David Turner, Robert Wilhelm, and Werner Lemberg.                      */
/*                                                                         */
/*  This file is part of the FreeType project, and may only be used,       */
/*  modified, and distributed under the terms of the FreeType project      */
/*  license, LICENSE.TXT.  By continuing to use, modify, or distribute     */
/*  this file you indicate that you have read the license and              */
/*  understand and accept it fully.                                        */
/*                                                                         */
/***************************************************************************/


#ifndef __T1TYPES_H__
#define __T1TYPES_H__


#include <ft2build.h>
#include FT_TYPE1_TABLES_H
#include FT_INTERNAL_POSTSCRIPT_NAMES_H
#include FT_INTERNAL_POSTSCRIPT_HINTS_H


FT_BEGIN_HEADER


  /*************************************************************************/
  /*************************************************************************/
  /*************************************************************************/
  /***                                                                   ***/
  /***                                                                   ***/
  /***              REQUIRED TYPE1/TYPE2 TABLES DEFINITIONS              ***/
  /***                                                                   ***/
  /***                                                                   ***/
  /*************************************************************************/
  /*************************************************************************/
  /*************************************************************************/


  /*************************************************************************/
  /*                                                                       */
  /* <Struct>                                                              */
  /*    T1_EncodingRec                                                     */
  /*                                                                       */
  /* <Description>                                                         */
  /*    A structure modeling a custom encoding.                            */
  /*                                                                       */
  /* <Fields>                                                              */
  /*    num_chars  :: The number of character codes in the encoding.       */
  /*                  Usually 256.                                         */
  /*                                                                       */
  /*    code_first :: The lowest valid character code in the encoding.     */
  /*                                                                       */
  /*    code_last  :: The highest valid character code in the encoding.    */
  /*                                                                       */
  /*    char_index :: An array of corresponding glyph indices.             */
  /*                                                                       */
  /*    char_name  :: An array of corresponding glyph names.               */
  /*                                                                       */
  typedef struct  T1_EncodingRecRec_
  {
    FT_Int       num_chars;
    FT_Int       code_first;
    FT_Int       code_last;

    FT_UShort*   char_index;
    FT_String**  char_name;

  } T1_EncodingRec, *T1_Encoding;


  typedef enum  T1_EncodingType_
  {
    T1_ENCODING_TYPE_NONE = 0,
    T1_ENCODING_TYPE_ARRAY,
    T1_ENCODING_TYPE_STANDARD,
    T1_ENCODING_TYPE_ISOLATIN1,
    T1_ENCODING_TYPE_EXPERT

  } T1_EncodingType;


  typedef struct  T1_FontRec_
  {
    PS_FontInfoRec   font_info;         /* font info dictionary */
    PS_PrivateRec    private_dict;      /* private dictionary   */
    FT_String*       font_name;         /* top-level dictionary */

    T1_EncodingType  encoding_type;
    T1_EncodingRec   encoding;

    FT_Byte*         subrs_block;
    FT_Byte*         charstrings_block;
    FT_Byte*         glyph_names_block;

    FT_Int           num_subrs;
    FT_Byte**        subrs;
    FT_Int*          subrs_len;

    FT_Int           num_glyphs;
    FT_String**      glyph_names;       /* array of glyph names       */
    FT_Byte**        charstrings;       /* array of glyph charstrings */
    FT_Int*          charstrings_len;

    FT_Byte          paint_type;
    FT_Byte          font_type;
    FT_Matrix        font_matrix;
    FT_Vector        font_offset;
    FT_BBox          font_bbox;
    FT_Long          font_id;

    FT_Int           stroke_width;

  } T1_FontRec, *T1_Font;


  typedef struct  CID_SubrsRec_
  {
    FT_UInt    num_subrs;
    FT_Byte**  code;

  } CID_SubrsRec, *CID_Subrs;


  /*************************************************************************/
  /*************************************************************************/
  /*************************************************************************/
  /***                                                                   ***/
  /***                                                                   ***/
  /***                ORIGINAL T1_FACE CLASS DEFINITION                  ***/
  /***                                                                   ***/
  /***                                                                   ***/
  /*************************************************************************/
  /*************************************************************************/
  /*************************************************************************/


  /*************************************************************************/
  /*                                                                       */
  /* This structure/class is defined here because it is common to the      */
  /* following formats: TTF, OpenType-TT, and OpenType-CFF.                */
  /*                                                                       */
  /* Note, however, that the classes TT_Size, TT_GlyphSlot, and TT_CharMap */
  /* are not shared between font drivers, and are thus defined normally in */
  /* `ttobjs.h'.                                                           */
  /*                                                                       */
  /*************************************************************************/

  typedef struct T1_FaceRec_*   T1_Face;
  typedef struct CID_FaceRec_*  CID_Face;


  typedef struct  T1_FaceRec_
  {
    FT_FaceRec     root;
    T1_FontRec     type1;
    const void*    psnames;
    const void*    psaux;
    const void*    afm_data;
    FT_CharMapRec  charmaprecs[2];
    FT_CharMap     charmaps[2];
    PS_Unicodes    unicode_map;

    /* support for Multiple Masters fonts */
    PS_Blend       blend;
    
    /* since FT 2.1 - interface to PostScript hinter */
    const void*    pshinter;

  } T1_FaceRec;


  /* CID is always smaller then 0x10000 */
  /* We must define other types of segments */
  /* when we want to use the bfrange operater (seg32to32) and */
  /* the UTF8 encoding with the Unicode extension B (seg64to32) */

  /* CMap_Seg32to16 */
  typedef struct  CID_CMapSeg32_
  {
    FT_ULong  start_char_code;
    FT_ULong  end_char_code;

    FT_UShort  start_glyph_id;
    FT_Byte id_incremental;  /* It is 1 (cidrange) or 0 (notdefrange) */

  } CID_CMapSeg32Rec, *CID_CMapSeg32;

  /* reduce cmap size */
  /* CMap_Seg16 */
  typedef struct  CID_CMapSeg16_
  {
    FT_UShort  start_char_code;
    FT_UShort  end_char_code;

    FT_UShort  start_glyph_id;
    FT_Byte id_incremental;  /* It is 1 (cidrange) or 0 (notdefrange) */

  } CID_CMapSeg16Rec, *CID_CMapSeg16;

  typedef struct CID_CMapCodeRangeRec_ *CID_CMapCodeRange;
  typedef struct CID_CMapTableRec_ *CID_CMapTable;

  /* redefine later? */
  typedef FT_UInt
  (*CID_CharMap_Func)( CID_CMapCodeRange charmap,
                      FT_ULong char_code );


  typedef struct CID_CMapCodeRangeRec_
  {
    FT_ULong start; /* srcCodeLo in Adobe's spec */
    FT_ULong end;   /* srcCodeHi */

    /* <00> 1byte, <0000> 2byte */
    FT_Byte byte;
    FT_ULong   num_segments;

    union
    {
      CID_CMapSeg32 seg32;
      CID_CMapSeg16 seg16;
      /* num_glyphs == startCharCode - endCharCode + 1 */
      FT_UShort* table16;
    } u;

    CID_CharMap_Func  get_index;

  } CID_CMapCodeRangeRec;

  typedef struct CID_CMapTableRec_
  {
    /* cidcmap_info */
    FT_String* name;
    FT_Fixed   version;
    FT_Int type;

    /* CIDSystemInfo */
    FT_String* registry;
    FT_String* ordering;
    FT_Int supplement;
    /* end CIDSystemInfo */

    FT_ULong uid_offset;

    FT_Int num_xuid;
    FT_ULong xuid[16];

    /* Is this type valid? */
    FT_UInt write_mode;
    /* cidcmap_info end */

    /* Is (0 <= num_parent <= 1) always true? */
    FT_String* parent_name;
    CID_CMapTable parent;

    FT_ULong num_code_ranges;
    CID_CMapCodeRange code_ranges;

  } CID_CMapTableRec;


  typedef struct CID_CMapRec_ CID_CMapRec,*CID_CMap;

  typedef struct  CID_FaceRec_
  {
    FT_FaceRec       root;
    void*            psnames;
    void*            psaux;
    CID_FaceInfoRec  cid;
    void*            afm_data;
    CID_Subrs        subrs;
    CID_CMap*     charmaps;
    
    /* since FT 2.1 - interface to PostScript hinter */
    void*            pshinter;

  } CID_FaceRec;


FT_END_HEADER

#endif /* __T1TYPES_H__ */


/* END */
